package dashboard

import (
	"net/http"

	"github.com/zeromicro/go-zero/rest/httpx"

	"wechat-api/internal/logic/dashboard"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
)

// swagger:route post /dashboard/labels dashboard GetLabels
//
// get labels | 获取图表数据
//
// get labels | 获取图表数据
//
// Parameters:
//  + name: body
//    require: true
//    in: body
//    type: LabelsReq
//
// Responses:
//  200: LabelsResp

func GetLabelsHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var req types.LabelsReq
		if err := httpx.Parse(r, &req, true); err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
			return
		}

		l := dashboard.NewGetLabelsLogic(r.Context(), svcCtx)
		resp, err := l.GetLabels(&req)
		if err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
		} else {
			httpx.OkJsonCtx(r.Context(), w, resp)
		}
	}
}
